//this file is part of xMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.xmule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifdef PRECOMP
#	include "xmule-headers.h"
#else
#	include "AddFileThread.h"
#	include "ChatWnd.h"
#	include "ClientList.h"
#	include "ClientUDPSocket.h"
#	include "config.h"
#	include "DownloadQueue.h"
#	include "ED2KLink.h"
#	include "FriendList.h"
#	include "KnownFile.h"
#	include "KnownFileList.h"
#	include "ListenSocket.h"
#	include "muuli_wdr.h"
#	include "opcodes.h"
#	include "PartFile.h"
#	include "PreferencesDlg.h"
#	include "resource.h"
#	include "SearchDlg.h"
#	include "ServerList.h"
#	include "ServerSocket.h"
#	include "ServerWnd.h"
#	include "SharedFileList.h"
#	include "SharedFilesWnd.h"
#	include "sockets.h"
#	include "StatisticsDlg.h"
#	include "SysTray.h"
#	include "TransferWnd.h"
#	include "UDPSocket.h"
#	include "UploadQueue.h"
#	include "xmule.h"
#	include "xmuleDlg.h"
#endif

#include <errno.h>
#include <math.h>
#include <signal.h>

#include <wx/socket.h>
#include <wx/splash.h>
#include <wx/toolbar.h>
#include <wx/utils.h>

#include "pixmaps/mule_TrayIcon.ico.xpm"
#include "pixmaps/mule_Tr_yellow.ico.xpm"
#include "pixmaps/mule_Tr_grey.ico.xpm"

// connection images

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

#if wxMINOR_VERSION > 4
#	define CLOSEBOX wxCLOSE_BOX
#else
#	define CLOSEBOX 0
#endif

#if 0
#define ID_BUTTONCONNECT 42110
#define ID_BUTTONSERVERS 42111
#define ID_BUTTONSEARCH 42112
#define ID_BUTTONTRANSFER 42113
#define ID_BUTTONPREFERENCES 42114
#define ID_BUTTONSHARED 42115
#define ID_BUTTONSTATS 42116
#endif

#define ID_UQTIMER 59742
#define TM_DNSDONE 17851

BEGIN_EVENT_TABLE(CxmuleDlg,wxFrame)
  EVT_SOCKET(ID_SOKETTI, CxmuleDlg::socketHandler)
  EVT_TOOL(ID_BUTTONSERVERS,CxmuleDlg::btnServers)
  EVT_TOOL(ID_BUTTONSEARCH,CxmuleDlg::btnSearch)
  EVT_TOOL(ID_BUTTONTRANSFER,CxmuleDlg::btnTransfer)
  EVT_TOOL(ID_BUTTONPREFERENCES,CxmuleDlg::btnPreferences)
  EVT_TOOL(ID_BUTTONCONNECT,CxmuleDlg::OnBnConnect)
  EVT_TOOL(ID_BUTTONSHARED,CxmuleDlg::OnBnShared)
  EVT_TOOL(ID_BUTTONSTATISTICS,CxmuleDlg::OnBnStats)
  EVT_TOOL(ID_BUTTONMESSAGES,CxmuleDlg::OnBnMessages)
  EVT_TIMER(ID_UQTIMER,CxmuleDlg::OnUQTimer)
  EVT_TIMER(4322,CxmuleDlg::OnUDPTimer)
  EVT_TIMER(4333,CxmuleDlg::OnSocketTimer)
  EVT_MENU(TM_FINISHEDHASHING,CxmuleDlg::OnFinishedHashing)
  EVT_MENU(TM_DNSDONE,CxmuleDlg::OnDnsDone)
  EVT_CLOSE(CxmuleDlg::OnClose)
  EVT_MENU(TM_HASHTHREADFINISHED,CxmuleDlg::OnHashingShutdown)
END_EVENT_TABLE()

void CxmuleDlg::OnDnsDone(wxCommandEvent& evt)
{
  CUDPSocket* socket=(CUDPSocket*)evt.GetClientData();
  struct sockaddr_in *si=(struct sockaddr_in*)evt.GetInt();
  socket->DnsLookupDone(si);
}

void CxmuleDlg::OnFinishedHashing(wxCommandEvent& evt)
{
  //wxMutexGuiEnter();

  CKnownFile* result = (CKnownFile*)evt.GetClientData();//(CKnownFile*)lParam;
  if (evt.GetInt()){
    CPartFile* requester = (CPartFile*)evt.GetInt(); //wParam;
    requester->PartFileHashFinished(result);
  }
  else{
    if(theApp.sharedfiles->filelist->SafeAddKFile(result))
       theApp.sharedfiles->SafeAddKFile(result);
    else
       delete result;
  }

  //wxMutexGuiLeave();
  return;
}

void muleDefaultToolbar( wxToolBar *parent )
{
    parent->SetMargins( 2, 2 );

    // First of all, create bar with the Disconnect button
    parent->AddTool( ID_BUTTONCONNECT, _("Connect"),connButImg( 0 ), _("Connect to server") );
    parent->AddSeparator();
    parent->AddCheckTool( ID_BUTTONSERVERS, _("Servers"),xmuleDlgImages( 3 ), wxNullBitmap,  _("Server list") );
    parent->AddCheckTool( ID_BUTTONSEARCH, _("Search"),xmuleDlgImages( 2 ), wxNullBitmap, _("Search files") );
    parent->AddCheckTool( ID_BUTTONTRANSFER, _("Transfer"),xmuleDlgImages( 1 ), wxNullBitmap, _("File transfer") );
    parent->AddCheckTool( ID_BUTTONSHARED, _("Shared Files"),xmuleDlgImages( 18 ), wxNullBitmap,_("Show shared files") );
    parent->AddCheckTool( ID_BUTTONMESSAGES, _("Messages"),xmuleDlgImages( 21 ), wxNullBitmap,  _("Messages") );
//    parent->AddCheckTool( ID_BUTTONIRC, _("IRC"),xmuleDlgImages( 19 ), wxNullBitmap, _("Internet Relay Chat")); 
    parent->AddCheckTool( ID_BUTTONSTATISTICS, _("Statistics"),xmuleDlgImages( 4 ), wxNullBitmap, _("Show current statistics"));
    parent->AddSeparator();
    parent->AddTool( ID_BUTTONPREFERENCES, _("Preferences"),xmuleDlgImages( 5 ), _("Modify preferences") );
    parent->Realize();
}

// CxmuleDlg Dialog

CxmuleDlg::CxmuleDlg(wxWindow* pParent, wxString title) 
  : wxFrame(pParent,CxmuleDlg::IDD,title,wxDefaultPosition,wxSize(800,520),
	    wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU|wxDIALOG_NO_PARENT|wxTHICK_FRAME|wxMINIMIZE_BOX|wxMAXIMIZE_BOX|CLOSEBOX)
{
  theApp.xmuledlg = this;

  m_app_state=APP_STATE_STARTING;
  srand(time(NULL));

  // get rid of sigpipe
  signal(SIGPIPE,SIG_IGN);
  
  //: CTrayDialog(CxmuleDlg::IDD, pParent){
  wxSizer* dlg=muleDlg(this,false,true);

  serverwnd=new CServerWnd(this);
  searchwnd=new CSearchDlg(this);
  transferwnd=new CTransferWnd(this);
  preferenceswnd=new CPreferencesDlg(this,theApp.glob_prefs);
  sharedfileswnd=new CSharedFilesWnd(this);
  statisticswnd=new CStatisticsDlg(this);
  chatwnd=new CChatWnd(this);

  transicons[0]=dlStatusImages(0);
  transicons[1]=dlStatusImages(1);
  transicons[2]=dlStatusImages(2);
  transicons[3]=dlStatusImages(3);

#define ID_BITMAPBUTTON (-1)

#if 0
  // create toolbar at the top
  wxBitmap* tikubmp=new wxBitmap(connButImg(0));
  mmMultiButton* tikutest=new mmMultiButton((wxWindow*)this,ID_BUTTONCONNECT,
					    wxT("Connect"),*tikubmp,wxDefaultPosition,
					    wxSize(70,70),mmMB_AUTODRAW);
  buttonToolbar->Add(tikutest,0,wxALIGN_LEFT|wxALL,5);

  wxStaticLine* fufusep=new wxStaticLine((wxWindow*)this,-1,wxDefaultPosition,wxDefaultSize,wxSL_VERTICAL);
  buttonToolbar->Add(fufusep,0,wxGROW|wxALIGN_LEFT|wxALL,5);

  tikubmp=new wxBitmap(xmuleDlgImages(3));
  tikutest=new mmMultiButton((wxWindow*)this,ID_BUTTONSERVERS,wxT("Servers"),*tikubmp,wxDefaultPosition,
					    wxSize(70,70),mmMB_AUTODRAW|mmMB_TOGGLE);
  buttonToolbar->Add(tikutest,0,wxALIGN_CENTRE|wxALL,5);
  tikutest->SetToggleDown(true);
  lastbutton=ID_BUTTONSERVERS;

  tikubmp=new wxBitmap(xmuleDlgImages(1));
  tikutest=new mmMultiButton((wxWindow*)this,ID_BUTTONTRANSFER,wxT("Transfer"),*tikubmp,wxDefaultPosition,
					    wxSize(70,70),mmMB_AUTODRAW|mmMB_TOGGLE);
  buttonToolbar->Add(tikutest,0,wxALIGN_CENTRE|wxALL,5);

  tikubmp=new wxBitmap(xmuleDlgImages(2));
  tikutest=new mmMultiButton((wxWindow*)this,ID_BUTTONSEARCH,wxT("Search"),*tikubmp,wxDefaultPosition,
					    wxSize(70,70),mmMB_AUTODRAW|mmMB_TOGGLE);
  buttonToolbar->Add(tikutest,0,wxALIGN_CENTRE|wxALL,5);

  tikubmp=new wxBitmap(xmuleDlgImages(18));
  tikutest=new mmMultiButton((wxWindow*)this,ID_BUTTONSHARED,wxT("Shared Files"),*tikubmp,wxDefaultPosition,
					    wxSize(70,70),mmMB_AUTODRAW|mmMB_TOGGLE);
  buttonToolbar->Add(tikutest,0,wxALIGN_CENTRE|wxALL,5);

  tikubmp=new wxBitmap(xmuleDlgImages(21));
  tikutest=new mmMultiButton((wxWindow*)this,ID_BITMAPBUTTON,wxT("Messages"),*tikubmp,wxDefaultPosition,
					    wxSize(70,70),mmMB_AUTODRAW);
  buttonToolbar->Add(tikutest,0,wxALIGN_CENTRE|wxALL,5);

  tikubmp=new wxBitmap(xmuleDlgImages(19));
  tikutest=new mmMultiButton((wxWindow*)this,ID_BITMAPBUTTON,wxT("IRC"),*tikubmp,wxDefaultPosition,
					    wxSize(70,70),mmMB_AUTODRAW);
  buttonToolbar->Add(tikutest,0,wxALIGN_CENTRE|wxALL,5);

  tikubmp=new wxBitmap(xmuleDlgImages(4));
  tikutest=new mmMultiButton((wxWindow*)this,ID_BUTTONSTATS,wxT("Statistics"),*tikubmp,wxDefaultPosition,
					    wxSize(70,70),mmMB_AUTODRAW|mmMB_TOGGLE);
  buttonToolbar->Add(tikutest,0,wxALIGN_CENTRE|wxALL,5);

  fufusep=new wxStaticLine((wxWindow*)this,-1,wxDefaultPosition,wxDefaultSize,wxSL_VERTICAL);
  buttonToolbar->Add(fufusep,0,wxGROW|wxALIGN_LEFT|wxALL,5);

  tikubmp=new wxBitmap(xmuleDlgImages(5));
  tikutest=new mmMultiButton((wxWindow*)this,ID_BUTTONPREFERENCES,wxT("Preferences"),*tikubmp,wxDefaultPosition,
					    wxSize(70,70),mmMB_AUTODRAW);
  buttonToolbar->Add(tikutest,0,wxALIGN_RIGHT|wxALL,5);
#endif
  wxToolBar* newbar=new wxToolBar(this,-1,wxDefaultPosition,wxDefaultSize,wxTB_HORIZONTAL|wxNO_BORDER|wxTB_TEXT);
  muleDefaultToolbar(newbar);
  //buttonToolbar->Add(newbar,0,wxGROW|wxALL,5);
  //buttonToolbar->Layout();
  m_wndToolbar=newbar;

  // initially server page is visible
  m_wndToolbar->ToggleTool(ID_BUTTONSERVERS,TRUE);
  lastbutton=ID_BUTTONSERVERS;

  // these are not implemented
  m_wndToolbar->EnableTool(ID_BUTTONIRC,FALSE);
  //m_wndToolbar->EnableTool(ID_BUTTONMESSAGES,FALSE);
  dlg->Show(this,TRUE);

  contentSizer->Add(serverwnd,1,wxALIGN_LEFT|wxEXPAND|wxALL,5);
  serverwnd->Show(FALSE);
  activewnd=serverwnd;
  searchwnd->Show(FALSE);
  transferwnd->Show(FALSE);
  sharedfileswnd->Show(FALSE);
  statisticswnd->Show(FALSE);
  chatwnd->Show(FALSE);

  SetActiveDialog(serverwnd);
  m_nActiveDialog = 1;
#if 0
  serverwnd.Show(FALSE);
  contentSizer->Remove(&serverwnd);
  contentSizer->Add(&searchwnd,1,wxALIGN_LEFT|wxEXPAND|wxALL,5);
  searchwnd.Show(TRUE);
  activewnd=&searchwnd;
#endif

  // can't init now
  //theApp.serverlist->Init();

#if 0
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	ready = false;
	lastuprate = 0;
	lastdownrate = 0;
	connicons[0] = (HICON)::LoadImage(AfxGetInstanceHandle(),MAKEINTRESOURCE(IDI_NOTCONNECTED),IMAGE_ICON,16,16,0);
	connicons[1] = (HICON)::LoadImage(AfxGetInstanceHandle(),MAKEINTRESOURCE(IDI_CONNECTED),IMAGE_ICON,16,16,0);
	connicons[2] = (HICON)::LoadImage(AfxGetInstanceHandle(),MAKEINTRESOURCE(IDI_CONNECTEDHIGH),IMAGE_ICON,16,16,0);
	transicons[0] = (HICON)::LoadImage(AfxGetInstanceHandle(),MAKEINTRESOURCE(IDI_UPDOWN),IMAGE_ICON,16,16,0);
	transicons[1] = (HICON)::LoadImage(AfxGetInstanceHandle(),MAKEINTRESOURCE(IDI_UP0DOWN1),IMAGE_ICON,16,16,0);
	transicons[2] = (HICON)::LoadImage(AfxGetInstanceHandle(),MAKEINTRESOURCE(IDI_UP1DOWN0),IMAGE_ICON,16,16,0);
	transicons[3] = (HICON)::LoadImage(AfxGetInstanceHandle(),MAKEINTRESOURCE(IDI_UP1DOWN1),IMAGE_ICON,16,16,0);
	imicons[0] = 0;
	imicons[1] = (HICON)::LoadImage(AfxGetInstanceHandle(),MAKEINTRESOURCE(IDI_MESSAGE),IMAGE_ICON,16,16,0);
	imicons[2] = (HICON)::LoadImage(AfxGetInstanceHandle(),MAKEINTRESOURCE(IDI_MPENDING),IMAGE_ICON,16,16,0);
	sourceTrayIcon= (HICON)LoadImage(AfxGetInstanceHandle() ,MAKEINTRESOURCE(IDR_TRAYICON),IMAGE_ICON,16,16,LR_DEFAULTCOLOR);
	sourceTrayIconGrey= (HICON)LoadImage(AfxGetInstanceHandle() ,MAKEINTRESOURCE(IDR_TRAYICON_GREY),IMAGE_ICON,16,16,LR_DEFAULTCOLOR);
	usericon = (HICON)::LoadImage(AfxGetInstanceHandle(),MAKEINTRESOURCE(IDI_USER),IMAGE_ICON,16,16,0);
#endif
/* Razor 1a - Modif by MikaelB
   Set client size and position */

  // Create a config base for saving razor preferences
  wxConfigBase *config = wxConfigBase::Get();
  // If config haven't been created
  if (config == NULL)
    // exit without saving
    return;

  // The section where to save in in file
  wxString section = "/Razor_Preferences/";

  // Main window location and size
  int x1,y1,x2,y2;


  // Saving window size and position
  x1 = config->Read(_T(section+"MAIN_X_POS"), 0l);
  y1 = config->Read(_T(section+"MAIN_Y_POS"), 0l);
  x2 = config->Read(_T(section+"MAIN_X_SIZE"), 0l);
  y2 = config->Read(_T(section+"MAIN_Y_SIZE"), 0l);
  split_pos = config->Read(_T(section+"SPLITTER_POS"), 0l);
  printf("split read, found : %u\n",split_pos);

  // If x1 and y1 != 0
  if((x1 != 0) && (y1 != 0))
    // Redefine location
    Move(x1,y1);

  // If x2 and y2 != 0
  if((x2 != 0) && (y2 != 0))
    // Redefine size
    SetClientSize(x2,y2);

/* End modif */

  // Modified tool bar integration... should not have any problems about cut Preferences button
  SetToolBar(newbar);

  CAddFileThread::Setup();
}

class QueryDlg : public wxDialog {
public:
  QueryDlg(wxWindow* parent) : wxDialog(parent,21373,_("Desktop integration"),wxDefaultPosition,
					wxDefaultSize,wxDEFAULT_DIALOG_STYLE|wxSYSTEM_MENU)
  {
    wxSizer* content=desktopDlg(this,TRUE);
    content->Show(this,TRUE);
    Centre();
  };
protected:
  void OnOk(wxEvent& evt) {EndModal(0);};

  DECLARE_EVENT_TABLE();
};

BEGIN_EVENT_TABLE(QueryDlg,wxDialog)
  EVT_BUTTON(ID_OK,QueryDlg::OnOk)
END_EVENT_TABLE()

void CxmuleDlg::changeDesktopMode()
{
    QueryDlg query(this);
    query.ShowModal();
    // then determine what choice is selected
    int mode=0;
    if(wxStaticCast(query.FindWindowById(ID_GNOME2),wxRadioButton)->GetValue()) {
      mode=1;
    } else if(wxStaticCast(query.FindWindowById(ID_KDE3),wxRadioButton)->GetValue()) {
      mode=2;
    } else if(wxStaticCast(query.FindWindowById(ID_KDE2),wxRadioButton)->GetValue()) {
      mode=3;
    } else {
      mode=4;
    }
    theApp.glob_prefs->SetDesktopMode(mode);
}

void CxmuleDlg::createSystray(const wxString& title)
{
  // create the docklet (at this point we already have preferences!)
  if(theApp.glob_prefs->GetDesktopMode()==0) {    
    // ok, it's not set yet.
    changeDesktopMode();
  }
  m_wndTaskbarNotifier=new CSysTray(this,theApp.glob_prefs->GetDesktopMode(),title);
}

extern void TimerProc();
void CxmuleDlg::OnUQTimer(wxTimerEvent& evt)
{
  // call timerproc
  if(!IsRunning())
    return;

  TimerProc();
}

void CxmuleDlg::InitDialog()
{
  theApp.serverlist->Init();
}

bool CxmuleDlg::IsRunning()
{
  return (m_app_state == APP_STATE_RUNNING);
}

void CxmuleDlg::OnBnStats(wxEvent& ev)
{
  //mmMultiButton* butn=wxStaticCast(FindWindowById(lastbutton),mmMultiButton);
  if(lastbutton == ID_BUTTONSTATISTICS)
    //butn->SetToggleDown(true);
    m_wndToolbar->ToggleTool(lastbutton,TRUE);
  else
    {    
      //butn->SetToggleDown(false);
      m_wndToolbar->ToggleTool(lastbutton,FALSE);
      lastbutton=ID_BUTTONSTATISTICS;
      SetActiveDialog(statisticswnd);
      m_nActiveDialog = 7;
    }
  //UpdateBar();
}

void CxmuleDlg::OnBnShared(wxEvent& ev)
{
  //mmMultiButton* butn=wxStaticCast(FindWindowById(lastbutton),mmMultiButton);
  if(lastbutton == ID_BUTTONSHARED)
    //butn->SetToggleDown(true);
    m_wndToolbar->ToggleTool(lastbutton,TRUE);
  else
    {    
      //butn->SetToggleDown(false);
      m_wndToolbar->ToggleTool(lastbutton,FALSE);
      lastbutton=ID_BUTTONSHARED;
      SetActiveDialog(sharedfileswnd);
      m_nActiveDialog = 4;
    }
  // UpdateBar();
}

void CxmuleDlg::btnServers(wxEvent& ev)
{
  //mmMultiButton* butn=wxStaticCast(FindWindowById(lastbutton),mmMultiButton);
  if(lastbutton == ID_BUTTONSERVERS)
    //butn->SetToggleDown(true);
    m_wndToolbar->ToggleTool(lastbutton,TRUE);
  else
    {    
      //butn->SetToggleDown(false);
      m_wndToolbar->ToggleTool(lastbutton,FALSE);
      lastbutton=ID_BUTTONSERVERS;
      SetActiveDialog(serverwnd);
      m_nActiveDialog = 1;
    }
  // UpdateBar();
}

void CxmuleDlg::btnSearch(wxEvent& ev)
{
  //mmMultiButton* butn=wxStaticCast(FindWindowById(lastbutton),mmMultiButton);
  if(lastbutton == ID_BUTTONSEARCH)
    //butn->SetToggleDown(true);
    m_wndToolbar->ToggleTool(lastbutton,TRUE);
  else
    {    
      //butn->SetToggleDown(false);
      m_wndToolbar->ToggleTool(lastbutton,FALSE);
      lastbutton=ID_BUTTONSEARCH;
      SetActiveDialog(searchwnd);
      m_nActiveDialog = 3;
    }
  // UpdateBar();
}

void CxmuleDlg::btnTransfer(wxEvent& ev)
{
  //mmMultiButton* butn=wxStaticCast(FindWindowById(lastbutton),mmMultiButton);
  if(lastbutton == ID_BUTTONTRANSFER)
    //butn->SetToggleDown(true);
    m_wndToolbar->ToggleTool(lastbutton,TRUE);
  else
    {    
      //butn->SetToggleDown(false);
      m_wndToolbar->ToggleTool(lastbutton,FALSE);
      lastbutton=ID_BUTTONTRANSFER;
      SetActiveDialog(transferwnd);
      m_nActiveDialog = 2;
    }
  // Set splitter position
  wxSplitterWindow* split=(wxSplitterWindow*)FindWindow("splitterWnd");
  split->SetSashPosition(split_pos, true);
  // UpdateBar();
}

void CxmuleDlg::OnBnMessages(wxEvent& ev)
{
  //mmMultiButton* butn=wxStaticCast(FindWindowById(lastbutton),mmMultiButton);
  if(lastbutton == ID_BUTTONMESSAGES)
    //butn->SetToggleDown(true);
    m_wndToolbar->ToggleTool(lastbutton,TRUE);
  else
    {    
      //butn->SetToggleDown(false);
      m_wndToolbar->ToggleTool(lastbutton,FALSE);
      lastbutton=ID_BUTTONMESSAGES;
      SetActiveDialog(chatwnd);
      m_nActiveDialog = 5;
    }
  // UpdateBar();
}

void CxmuleDlg::btnPreferences(wxEvent& ev)
{
  // do preferences
  preferenceswnd->ShowModal();
}

void CxmuleDlg::socketHandler(wxSocketEvent& event)
{
  if(!event.GetSocket())
    return;

  if(!IsRunning()) {
    // we are not mentally ready to receive anything
    return;
  }

  if(event.GetSocket()->IsKindOf(CLASSINFO(CListenSocket))) {
    CListenSocket* soc=(CListenSocket*)event.GetSocket();
    switch(event.GetSocketEvent()) {
    case wxSOCKET_CONNECTION:
      soc->OnAccept(0);
      break;
    default:
      // shouldn't get other than connection events...
      break;
    }
    return;
  }

  if(event.GetSocket()->IsKindOf(CLASSINFO(CClientReqSocket))) {
    CClientReqSocket* soc=(CClientReqSocket*)event.GetSocket();
    //printf("request at clientreqsocket\n");
    switch(event.GetSocketEvent()) {
    case wxSOCKET_LOST:
      soc->OnError(errno);
      break;
    case wxSOCKET_INPUT:
      soc->OnReceive(0);
      break;
    case wxSOCKET_OUTPUT:
      soc->OnSend(0);
      break;
    default:
      // connection requests should not arrive here..
      break;
    }
    return;
  }

  if(event.GetSocket()->IsKindOf(CLASSINFO(CUDPSocket))) {
    CUDPSocket* soc=(CUDPSocket*)event.GetSocket();
    switch(event.GetSocketEvent()) {
    case wxSOCKET_INPUT:
      soc->OnReceive(0);
      break;
    default:
      break;
    }
    return;
  }

  if(event.GetSocket()->IsKindOf(CLASSINFO(CServerSocket))) {
    CServerSocket* soc=(CServerSocket*)event.GetSocket();
    switch(event.GetSocketEvent()) {
    case wxSOCKET_CONNECTION:
      soc->OnConnect(0);
      break;
    case wxSOCKET_LOST:
      soc->OnError(errno);
      break;
    case wxSOCKET_INPUT:
      soc->OnReceive(0);
      break;
    case wxSOCKET_OUTPUT:
      soc->OnSend(0);
      break;
    default:
      break;
    }
    return;
  }

  if(event.GetSocket()->IsKindOf(CLASSINFO(CClientUDPSocket))) {
    CClientUDPSocket* soc=(CClientUDPSocket*)event.GetSocket();
    switch(event.GetSocketEvent()) {
    case wxSOCKET_INPUT:
      soc->OnReceive(0);
      break;
    case wxSOCKET_OUTPUT:
      soc->OnSend(0);
      break;
    default:
      break;
    }
    return;
  }

  printf("*** SHOULD NOT END UP HERE\n");
  printf("** class is %s\n",event.GetSocket()->GetClassInfo()->GetClassName());
}

CxmuleDlg::~CxmuleDlg()
{
	printf("Shutting down xMule...\n");

#if 0
	::DestroyIcon(m_hIcon);
	::DestroyIcon(connicons[0]);
	::DestroyIcon(connicons[1]);
	::DestroyIcon(connicons[2]);
	::DestroyIcon(transicons[0]);
	::DestroyIcon(transicons[1]);
	::DestroyIcon(transicons[2]);
	::DestroyIcon(transicons[3]);
	::DestroyIcon(imicons[1]);
	::DestroyIcon(imicons[2]);
	::DestroyIcon(sourceTrayIcon);
	::DestroyIcon(sourceTrayIconGrey);
	::DestroyIcon(usericon);
#endif
/* Razor 1a - Modif by MikaelB
   Save client size and position */

  // Create a config base for saving razor preferences
  wxConfigBase *config = wxConfigBase::Get();
  // If config haven't been created
  if (config == NULL)
    // exit without saving
    return;

  // The section where to save in in file
  wxString section = "/Razor_Preferences/";

  // Main window location and size
  int x1,y1,x2,y2;
  GetPosition(&x1, &y1);
  GetSize(&x2,&y2);

  // Saving window size and position
  config->Write(_T(section+"MAIN_X_POS"), (long) x1);
  config->Write(_T(section+"MAIN_Y_POS"), (long) y1);
  config->Write(_T(section+"MAIN_X_SIZE"), (long) x2);
  config->Write(_T(section+"MAIN_Y_SIZE"), (long) y2);
  
  // Saving sash position of splitter in transfer window
  wxSplitterWindow* split=(wxSplitterWindow*)FindWindow("splitterWnd");
  config->Write(_T(section+"SPLITTER_POS"), (long) split_pos);
  printf("split saved to : %u\n", split_pos);
	  
  config->Flush(true);

/* End modif */


	theApp.OnlineSig(true);

	// TODO: We want to free the memory used by the webserver
	//       but currently it does wait in accept().
	// delete theApp.webserver;
	delete theApp.serverlist; theApp.serverlist = NULL;
	delete theApp.searchlist; theApp.searchlist = NULL;
	delete theApp.clientcredits; theApp.clientcredits = NULL;
	// Destroying CDownloadQueue calls destructor for CPartFile
	// calling CSharedFileList::SafeAddKFile occasally.
	delete theApp.downloadqueue; theApp.downloadqueue = NULL;
	delete theApp.sharedfiles; theApp.sharedfiles = NULL;
	delete theApp.knownfiles; theApp.knownfiles = NULL;
	delete theApp.uploadqueue; theApp.uploadqueue = NULL;
	delete theApp.clientlist; theApp.clientlist = NULL;
	delete theApp.friendlist; theApp.friendlist = NULL;
	delete theApp.glob_prefs; theApp.glob_prefs = NULL;
	delete theApp.serverconnect; theApp.serverconnect = NULL;
}
#if 0
void CxmuleDlg::DoDataExchange(CDataExchange* pDX){
	CTrayDialog::DoDataExchange(pDX);
	
	// Masterbuttons
	DDX_Control(pDX, IDC_BUTTON2, m_btnConnect);
	DDX_Control(pDX, IDC_BN_DOWNLOADS, m_btnDownloads);
	DDX_Control(pDX, IDC_BN_SERVERS, m_btnServers);
	DDX_Control(pDX, IDC_BN_SEARCH, m_btnSearch);
	DDX_Control(pDX, IDC_BN_FILES, m_btnFiles);
	DDX_Control(pDX, IDC_BN_PREFERENCES, m_btnPreferences );
	DDX_Control(pDX, IDC_BN_MESSAGES, m_btnMessages );
	DDX_Control(pDX, IDC_BN_STATISTICS , m_btnStatistics );
	DDX_Control(pDX, IDC_BN_IRC, m_btnIrc );
}

BEGIN_MESSAGE_MAP(CxmuleDlg, CTrayDialog)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BUTTON2, OnBnClickedButton2)

	ON_BN_CLICKED(IDC_BN_DOWNLOADS , OnBnClicked2)
	ON_BN_CLICKED(IDC_BN_SERVERS , OnBnClicked3)
	ON_BN_CLICKED(IDC_BN_SEARCH , OnBnClicked4)
	ON_BN_CLICKED(IDC_BN_FILES , OnBnClicked5)
	ON_BN_CLICKED(IDC_BN_MESSAGES , OnBnClicked6)
	ON_BN_CLICKED(IDC_BN_STATISTICS, OnBnClicked7)
	ON_BN_CLICKED(IDC_BN_PREFERENCES , OnBnClicked8)
	ON_BN_CLICKED(IDC_BN_IRC , OnBnClicked9)

	ON_WM_SIZE()
	ON_MESSAGE(WM_COPYDATA, OnWMData)
	ON_MESSAGE(TM_FINISHEDHASHING,OnFileHashed)
	ON_WM_CLOSE()
	ON_COMMAND(MP_RESTORE, RestoreWindow)
	ON_COMMAND(MP_CONNECT, StartConnection)
	ON_COMMAND(MP_DISCONNECT, CloseConnection)
	ON_COMMAND(MP_EXIT, OnClose )
	ON_WM_CREATE()
END_MESSAGE_MAP()
#endif

// CxmuleDlg eventhandler

#if 0
BOOL CxmuleDlg::OnInitDialog(){	
	CTrayDialog::OnInitDialog();
	Localize();
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);
	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL){
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty()){
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	SetIcon(m_hIcon, TRUE);			
	SetIcon(m_hIcon, FALSE);

	// Setting up IconButtons
	m_btnConnect.SetIcon(IDI_BN_CONNECT);		m_btnConnect.SetAlign(CButtonST::ST_ALIGN_VERT);
	m_btnDownloads.SetIcon(IDI_BN_DOWNLOAD);	m_btnDownloads.SetAlign(CButtonST::ST_ALIGN_VERT);
	m_btnServers.SetIcon(IDI_BN_SERVER );		m_btnServers.SetAlign(CButtonST::ST_ALIGN_VERT);
	m_btnSearch.SetIcon(IDI_BN_SEARCH);			m_btnSearch.SetAlign(CButtonST::ST_ALIGN_VERT);
	m_btnFiles.SetIcon(IDI_BN_FILES);			m_btnFiles.SetAlign(CButtonST::ST_ALIGN_VERT);
	m_btnStatistics.SetIcon(IDI_BN_STATISTICS);	m_btnStatistics.SetAlign(CButtonST::ST_ALIGN_VERT);
	m_btnIrc.SetIcon(IDI_BN_MESSAGES);			m_btnIrc.SetAlign(CButtonST::ST_ALIGN_VERT);
	m_btnMessages.SetIcon(IDI_BN_MESSAGES);		m_btnMessages.SetAlign(CButtonST::ST_ALIGN_VERT);
	m_btnPreferences.SetIcon(IDI_BN_PREFERENCES);m_btnPreferences.SetAlign(CButtonST::ST_ALIGN_VERT);

	//set title
	char buffer[50];
	sprintf(buffer,"xMule v%s",VERSION);
	SetWindowText(buffer);

	//START - enkeyDEV(kei-kun) -TaskbarNotifier-
	m_wndTaskbarNotifier.Create(this);
	CEnBitmap m_imgTaskbar;
	VERIFY (m_imgTaskbar.LoadImage(IDR_TASKBAR,"GIF"));
	m_wndTaskbarNotifier.SetBitmap(&m_imgTaskbar, 255, 0, 255);
	m_wndTaskbarNotifier.SetTextFont("Arial",70,TN_TEXT_NORMAL,TN_TEXT_UNDERLINE);
	m_wndTaskbarNotifier.SetTextColor(RGB(255,255,255),RGB(255,255,255));		
	m_wndTaskbarNotifier.SetTextRect(CRect(5,45,m_wndTaskbarNotifier.m_nBitmapWidth-80, m_wndTaskbarNotifier.m_nBitmapHeight-35));
	//END - enkeyDEV(kei-kun) -TaskbarNotifier-

	// set statusbar
	statusbar.Create(WS_CHILD|WS_VISIBLE|CCS_BOTTOM|SBARS_SIZEGRIP,CRect(0,0,0,0), this, IDC_STATUSBAR) ;
	statusbar.EnableToolTips(true);
	CRect rect;
	statusbar.GetClientRect(&rect);
	int widths[5] = { rect.right-460, rect.right-350, rect.right-190,rect.right-25, -1 };
	statusbar.SetParts(5, widths);
	statusbar.SetIcon(1,usericon);
	statusbar.SetText(GetResString(IDS_UUSERS)+": 0",1,0);



	// create dialog pages
	preferenceswnd.SetPrefs(theApp.glob_prefs);
	serverwnd.Create(IDD_SERVER);
	sharedfileswnd.Create(IDD_FILES);
	searchwnd.Create(IDD_SEARCH);
	chatwnd.Create(IDD_CHAT);
	transferwnd.Create(IDD_TRANSFER);
	statisticswnd.Create(IDD_STATISTICS);
	ircwnd.Create(IDD_IRC);
	activewnd = &transferwnd;

	// set updateintervall of graphic rate display (in seconds)
	ShowConnectionState(false);

	// anchors
	AddAnchor(serverwnd,TOP_LEFT,BOTTOM_RIGHT);
	AddAnchor(transferwnd,TOP_LEFT,BOTTOM_RIGHT);
	AddAnchor(sharedfileswnd,TOP_LEFT,BOTTOM_RIGHT);
	AddAnchor(searchwnd,TOP_LEFT,BOTTOM_RIGHT);
	AddAnchor(chatwnd,TOP_LEFT,BOTTOM_RIGHT);
	AddAnchor(ircwnd,TOP_LEFT,BOTTOM_RIGHT);
	AddAnchor(statusbar,BOTTOM_LEFT,BOTTOM_RIGHT);
	AddAnchor(statisticswnd,TOP_LEFT,BOTTOM_RIGHT);
	AddAnchor(m_btnPreferences,TOP_RIGHT);
	ShowTransferRate();
	statisticswnd.ShowInterval();
	// tray icon
	TraySetMinimizeToTray(theApp.glob_prefs->GetMinTrayPTR());

	serverwnd.ShowWindow(SW_SHOW);
	activewnd = &serverwnd;

	theApp.serverlist->Init();
	ready = true;

	// start listining on edonkeyport
	if (!theApp.listensocket->StartListening())
		AddLogLine(false,GetResString(IDS_MAIN_SOCKETERROR),theApp.glob_prefs->GetPort());

	// ini. downloadqueue
	theApp.downloadqueue->Init();

	AddLogLine(true,PACKAGE_STRING);

	if (theApp.pendinglink){
		OnWMData(NULL,(LPARAM) &theApp.sendstruct);//changed by Cax2 28/10/02
		delete theApp.pendinglink;
	}

	if (theApp.glob_prefs->DoAutoConnect())
		OnBnClickedButton2();

	// Restore saved window placement
	WINDOWPLACEMENT wp;wp.length=sizeof(wp);
	wp=theApp.glob_prefs->GetEmuleWindowPlacement();
	SetWindowPlacement(&wp);

	// splashscreen
	if (theApp.glob_prefs->UseSplashScreen()){
		CSplashScreen splash(this);
		splash.DoModal();
	}


	return TRUE;
}

void CxmuleDlg::OnSysCommand(UINT nID, LPARAM lParam){
	if ((nID & 0xFFF0) == IDM_ABOUTBOX){
		CCreditsDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else{
		CTrayDialog::OnSysCommand(nID, lParam);
	}
}

void CxmuleDlg::OnPaint() {
	if (IsIconic()){
		CPaintDC dc(this);

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		dc.DrawIcon(x, y, m_hIcon);
	}
	else{
		CTrayDialog::OnPaint();
	}
}


HCURSOR CxmuleDlg::OnQueryDragIcon(){
	return static_cast<HCURSOR>(m_hIcon);
}

void CxmuleDlg::OnBnClickedButton2(){
  if (!theApp.serverconnect->IsConnected()) 
    //connect if not currently connected
	if (!theApp.serverconnect->IsConnecting()  ) StartConnection(); else {
		theApp.serverconnect->StopConnectionTry();
		ShowConnectionState(false);
	}
  else
    //disconnect if currently connected
    CloseConnection();
}
#endif

void CxmuleDlg::OnBnConnect(wxEvent& evt)
{
  if (!theApp.serverconnect->IsConnected()) 
    //connect if not currently connected
    if (!theApp.serverconnect->IsConnecting()  ) StartConnection(); else {
      theApp.serverconnect->StopConnectionTry();
      ShowConnectionState(false);
    }
  else {     //disconnect if currently connected
    CloseConnection();
  }
  // UpdateBar();
}

void CxmuleDlg::ResetLog(){
	logtext="";
	//serverwnd.logbox.SetWindowText(logtext);
	wxTextCtrl* ct=(wxTextCtrl*)serverwnd->FindWindowByName("logView");
	if(ct) {
	  ct->SetValue("");
	}
	// Delete log file aswell.
	wxRemoveFile(wxString::Format("%s/.xMule/logfile", getenv("HOME")));
}

void CxmuleDlg::ResetDebugLog(){
	logtext="";
	serverwnd->logbox.Clear(); //SetWindowText(logtext);
}

void CxmuleDlg::AddLogLine(bool addtostatusbar,const wxChar* line,...){
	char osDate[30],osTime[30]; //<<--9/21/02
	char temp[1060]; //<<--9/21/02
	char bufferline[1000];

	va_list argptr;
	va_start(argptr, line);
	vsnprintf(bufferline, 1000, line, argptr);
	va_end(argptr);
	if (addtostatusbar) {
	//  statusbar.SetStatusText(bufferline,0); //,0);
	  wxStaticText* text=(wxStaticText*)FindWindow("infoLabel");
	  text->SetLabel(bufferline);
	Layout();
	}
	
	time_t joskus=time(NULL);
	struct tm* nyt=localtime(&joskus);
	strftime(osDate,29,"%F",nyt);
	strftime(osTime,29,"%T",nyt);

	//strtime( osTime ); //<<--9/21/02
	//strdate( osDate ); //<<--9/21/02
	sprintf(temp,"%s %s: %s\n",osDate,osTime,bufferline);//<<--9/21/02 

	wxTextCtrl* ct=(wxTextCtrl*)serverwnd->FindWindowByName("logView");
	if(ct) {
	  ct->AppendText(temp);
	  //ct->ShowPosition(ct->GetLastPosition());
	  ct->ShowPosition(ct->GetValue().Length()-1);
	}
	// Write into log file
	wxString filename;
	filename = filename.Format("%s/.xMule/logfile", getenv("HOME"));
	wxFile file(filename, wxFile::write_append);
	if (file.IsOpened()) {
		file.Write(temp);
		file.Close();
	}
#if 0
	if (ready){
	  //serverwnd.logbox.SetWindowText(logtext);
	  serverwnd.logbox.Clear();
	  serverwnd.logbox.AppendText(logtext);
	  //serverwnd.logbox.LineScroll(serverwnd.logbox.GetLineCount());
	  serverwnd.logbox.ShowPosition(serverwnd.logbox.GetLastPosition());
	  ShowNotifier(bufferline, TBN_LOG, false);
	}
#endif
	//printf("***LOG: %s\n",logtext);
}

void CxmuleDlg::AddDebugLogLine(bool addtostatusbar,const wxChar* line,...){
#if 0
	if( !theApp.glob_prefs->GetVerbose() )
		return;
	char osDate[30],osTime[30]; //<<--9/21/02
	char temp[1060]; //<<--9/21/02
	char bufferline[1000];

	va_list argptr;
	va_start(argptr, line);
	_vsnprintf(bufferline, 1000, line, argptr);
	va_end(argptr);
	if (addtostatusbar)
		statusbar.SetText(bufferline,0,0);
	
	_strtime( osTime ); //<<--9/21/02
	_strdate( osDate ); //<<--9/21/02
	sprintf(temp,"%s %s: %s",osDate,osTime,bufferline);//<<--9/21/02 
	debuglog += temp; //<<--9/21/02
	debuglog += "\r\n";
	if (ready){
		serverwnd.debuglog.SetWindowText(debuglog);
		serverwnd.debuglog.LineScroll(serverwnd.debuglog.GetLineCount());
	}
#endif
}

void CxmuleDlg::AddServerMessageLine(char* line,...){
	wxString content;
	va_list argptr;
	char bufferline[500];
	va_start(argptr, line);
	vsnprintf(bufferline, 500, line, argptr);
	va_end(argptr);

	wxTextCtrl* cv=(wxTextCtrl*)serverwnd->FindWindowByName("serverInfo");
	if(cv) {
  	  cv->AppendText(wxString(bufferline)+wxString("\n"));
	  cv->ShowPosition(cv->GetValue().Length()-1);
	}
	//serverwnd.servermsgbox.AppendText(CString(bufferline)+CString("\n"));
}

void CxmuleDlg::ShowConnectionState(bool connected){ShowConnectionState(connected,"");}

#define STAT_LOWID 1
#define STAT_HIGHID 2
#define STAT_CONNECTING 3
#define STAT_NOTCONNECTED 4

void CxmuleDlg::ShowConnectionState(bool connected, wxString server,bool iconOnly){
  static int __oldstatus=(-1);
  bool changed=false;

  serverwnd->UpdateMyInfo();

  if (connected){
    wxStaticBitmap* bmp=(wxStaticBitmap*)FindWindowByName("connImage");
    if(theApp.serverconnect->IsLowID()) {
      if(__oldstatus!=STAT_LOWID) {
	bmp->SetBitmap(connImages(1));	    
	__oldstatus=STAT_LOWID;
	changed=true;
      }
    } else {
      if(__oldstatus!=STAT_HIGHID) {
	bmp->SetBitmap(connImages(3));
	__oldstatus=STAT_HIGHID;
	changed=true;
      }
    }

    //butn->SetLabel(GetResString(IDS_MAIN_BTN_DISCONNECT));
    //butn->SetDefaultBitmap(connButImg(1));
    // we can't modify existing button. so we delete the old and create a new one
    if(changed) {
      m_wndToolbar->DeleteTool(ID_BUTTONCONNECT);
      m_wndToolbar->InsertTool(0,ID_BUTTONCONNECT,GetResString(IDS_MAIN_BTN_DISCONNECT),connButImg(1),GetResString(IDS_MAIN_BTN_DISCONNECT_TOOLTIP));
      m_wndToolbar->Realize();
    }

    /*
      this->GetDlgItem(IDC_BUTTON2)->SetWindowText(GetResString(IDS_MAIN_BTN_DISCONNECT));
      m_btnConnect.SetIcon(IDI_BN_DISCONNECT); m_btnConnect.SetTooltipText(GetResString(IDS_MAIN_BTN_DISCONNECT_TOOLTIP));
      
      if (theApp.serverconnect->IsLowID()) 
      statusbar.SetIcon(3,connicons[1]);
      else 
      statusbar.SetIcon(3,connicons[2]);
    */
    //statusbar.SetText(GetResString(IDS_MAIN_CONNECTEDTO) +CString(server),0,0);
    wxStaticText* tx=(wxStaticText*)FindWindow("infoLabel");
    tx->SetLabel(GetResString(IDS_MAIN_CONNECTEDTO)+wxString(server));
    ((wxStaticText*)FindWindow("connLabel"))->SetLabel(server);
    Layout();
    //statusbar.SetTipText(3,server);
    //statusbar.SetText(server,3,0);
  }
  else{
    
    wxStaticBitmap* bmp=(wxStaticBitmap*)FindWindowByName("connImage");
    
    if (theApp.serverconnect->IsConnecting()) {		  
      if(__oldstatus!=STAT_CONNECTING) {
	bmp->SetBitmap(connImages(0));
	//butn->SetLabel(GetResString(IDS_MAIN_BTN_CANCEL));
	//butn->SetDefaultBitmap(connButImg(2));
	// once again.. no modify
	m_wndToolbar->DeleteTool(ID_BUTTONCONNECT);
	m_wndToolbar->InsertTool(0,ID_BUTTONCONNECT,GetResString(IDS_MAIN_BTN_CANCEL),connButImg(2),
				 GetResString(IDS_MAIN_BTN_CANCEL_TOOLTIP));
	m_wndToolbar->Realize();
	__oldstatus=STAT_CONNECTING;
      }

      /*this->GetDlgItem(IDC_BUTTON2)->SetWindowText(GetResString(IDS_MAIN_BTN_CANCEL));
	m_btnConnect.SetIcon(IDI_BN_STOPCONNECTING); m_btnConnect.SetTooltipText(GetResString(IDS_MAIN_BTN_CANCEL_TOOLTIP));
	statusbar.SetIcon(3,connicons[0]);*/
      
      // EI INFOLABEL
      ((wxStaticText*)FindWindowByName("connLabel"))->SetLabel(GetResString(IDS_CONNECTING));
      Layout();
      //statusbar.SetText(GetResString(IDS_CONNECTING),3,0);
      //statusbar.SetTipText(3,"");
      ShowUserCount(0,0);
      
    } else {
      if(__oldstatus!=STAT_NOTCONNECTED) {
	bmp->SetBitmap(connImages(0));
	((wxStaticText*)FindWindowByName("connLabel"))->SetLabel(GetResString(IDS_NOTCONNECTED));
      Layout();
      //butn->SetLabel(GetResString(IDS_MAIN_BTN_CONNECT));
      //butn->SetDefaultBitmap(connButImg(0));
      // and again...
	m_wndToolbar->DeleteTool(ID_BUTTONCONNECT);
	m_wndToolbar->InsertTool(0,ID_BUTTONCONNECT,GetResString(IDS_MAIN_BTN_CONNECT),connButImg(0),
				 GetResString(IDS_CONNECTTOANYSERVER));
	m_wndToolbar->Realize();
	__oldstatus=STAT_NOTCONNECTED;
      }
      /*
	this->GetDlgItem(IDC_BUTTON2)->SetWindowText(GetResString(IDS_MAIN_BTN_CONNECT));
	m_btnConnect.SetIcon(IDI_BN_CONNECT);	m_btnConnect.SetTooltipText(GetResString(IDS_CONNECTTOANYSERVER));
	
	statusbar.SetIcon(3,connicons[0]);
	statusbar.SetText(GetResString(IDS_NOTCONNECTED),3,0);
      */
      ((wxStaticText*)FindWindow("connLabel"))->SetLabel(GetResString(IDS_NOTCONNECTED));
      Layout();
      AddLogLine(true,GetResString(IDS_DISCONNECTED));
      //statusbar.SetTipText(3,"");
      ShowUserCount(0,0);
    }
    
  }
}

void CxmuleDlg::ShowUserCount(uint32 user_toshow,uint32 file_toshow){
  uint32 totaluser, totalfile;
  totaluser = totalfile = 0;
  if( user_toshow || file_toshow )
    theApp.serverlist->GetUserFileStatus( totaluser, totalfile );
  char buffer[100];
  sprintf(buffer, "%s: %s(%s) | %s: %s(%s)", GetResString(IDS_UUSERS).GetData(), CastItoIShort(user_toshow).GetData(), CastItoIShort(totaluser).GetData(), GetResString(IDS_FILES).GetData(), CastItoIShort(file_toshow).GetData(), CastItoIShort(totalfile).GetData());
  wxStaticCast(FindWindow("userLabel"),wxStaticText)->SetLabel(buffer);
  Layout();
  //statusbar.SetText(buffer,1,0);
}

void CxmuleDlg::ShowMessageState(uint8 iconnr){
  //statusbar.SetIcon(4,imicons[iconnr]);
}

void CxmuleDlg::ShowTransferRate(bool foreceAll){
	char buffer[50];
	char buffer2[100];

	lastuprate = theApp.uploadqueue->GetDatarate();
	lastdownrate= theApp.downloadqueue->GetDatarate();
	int lastuprateoverhead = theApp.uploadqueue->GetUpDatarateOverhead();
	int lastdownrateoverhead = theApp.downloadqueue->GetDownDatarateOverhead();

	if( theApp.glob_prefs->ShowOverhead() )
		sprintf(buffer,GetResString(IDS_UPDOWN),(float)lastuprate/1024, (float)lastuprateoverhead/1024, (float)lastdownrate/1024, (float)lastdownrateoverhead/1024);
	else
		sprintf(buffer,GetResString(IDS_UPDOWNSMALL),(float)lastuprate/1024, (float)lastdownrate/1024);
	//statusbar.SetStatusText(buffer,2); //,0);
	((wxStaticText*)FindWindowByName("speedLabel"))->SetLabel(buffer);
	Layout();

	// set trayicon-icon
	int DownRateProcent=(int)ceil ( (lastdownrate/10.24)/ theApp.glob_prefs->GetMaxGraphDownloadRate());
	if (DownRateProcent>100) DownRateProcent=100;
	UpdateTrayIcon(DownRateProcent);

	if (theApp.serverconnect->IsConnected()) 
		sprintf(buffer2,wxString(_("xMule (%s | ")+GetResString(IDS_CONNECTED)+")").GetData(),buffer);
	else
		sprintf(buffer2,wxString(_("xMule (%s | ")+GetResString(IDS_DISCONNECTED)+")").GetData(),buffer);

	m_wndTaskbarNotifier->TraySetToolTip(buffer2);

	wxStaticBitmap* bmp=(wxStaticBitmap*)FindWindowByName("transferImg");
	if(lastuprate&&lastdownrate)
	  bmp->SetBitmap(transicons[3]);
	else if(lastuprate)
	  bmp->SetBitmap(transicons[2]);
	else if(lastdownrate)
	  bmp->SetBitmap(transicons[1]);
	else
	  bmp->SetBitmap(transicons[0]);
#if 0
	if (lastuprate && lastdownrate)
		statusbar.SetIcon(2,transicons[3]);
	else if (lastuprate)
		statusbar.SetIcon(2,transicons[2]);
	else if (lastdownrate)
		statusbar.SetIcon(2,transicons[1]);
	else
		statusbar.SetIcon(2,transicons[0]);
#endif
}

#if 0
void CxmuleDlg::OnCancel(){

if (*theApp.glob_prefs->GetMinTrayPTR()) {
  //TrayShow();
  //		ShowWindow(SW_HIDE);
	} else {
	  //	ShowWindow(SW_MINIMIZE);
	}

};
#endif

void CxmuleDlg::SetActiveDialog(wxWindow* dlg){
  activewnd->Show(FALSE);
  contentSizer->Remove(activewnd);
  contentSizer->Add(dlg,1,wxALIGN_LEFT|wxEXPAND|wxALL,5);
  dlg->Show(TRUE);
  activewnd=dlg;
  Layout();
}

#if 0
void CxmuleDlg::OnSize(UINT nType,int cx,int cy){
	CTrayDialog::OnSize(nType,cx,cy);
	CRect rect;
	statusbar.GetClientRect(&rect);
	int widths[5] = { rect.right-460, rect.right-350, rect.right-190,rect.right-25, -1 };
	statusbar.SetParts(5, widths);
}
#endif

extern void URLDecode(wxString& result, const char* buff);

#if 0
LRESULT CxmuleDlg::OnWMData(WPARAM wParam,LPARAM lParam){
	tagCOPYDATASTRUCT* data = (tagCOPYDATASTRUCT*)lParam;
	if (data->dwData == OP_ED2KLINK){
		FlashWindow(true);
		if( theApp.glob_prefs->IsBringToFront() ){
			if (IsIconic())
				ShowWindow(SW_SHOWNORMAL);
			else if (TrayHide()){
				ShowWindow(SW_SHOW);
			}
		else
			SetForegroundWindow();
		}
		try {
			CString link2;
			CString link;
			link2 = (char*)data->lpData;
			link2.Replace("%7c","|");
			URLDecode(link,(const char*)link2.GetBuffer());
			CED2KLink* pLink = CED2KLink::CreateLinkFromUrl(link);
			_ASSERT( pLink !=0 );
			switch (pLink->GetKind()) {
			case CED2KLink::kFile:
				{
					CED2KFileLink* pFileLink = pLink->GetFileLink();
					_ASSERT(pFileLink !=0);
					theApp.downloadqueue->AddFileLinkToDownload(pFileLink);
				}
				break;
			case CED2KLink::kServerList:
				{
					CED2KServerListLink* pListLink = pLink->GetServerListLink(); 
					_ASSERT( pListLink !=0 ); 
					CString strAddress = pListLink->GetAddress(); 
					if(strAddress.GetLength() != 0)
						serverwnd.UpdateServerMetFromURL(strAddress);
				}
				break;
			case CED2KLink::kServer:
				{
					CString defName;
					CED2KServerLink* pSrvLink = pLink->GetServerLink();
					_ASSERT( pSrvLink !=0 );
					in_addr host;
					host.S_un.S_addr = pSrvLink->GetIP();
					CServer* pSrv = new CServer(pSrvLink->GetPort(),inet_ntoa(host));
					_ASSERT( pSrv !=0 );
					pSrvLink->GetDefaultName(defName);
					pSrv->SetListName(defName.GetBuffer());
					if (!serverwnd.serverlistctrl.AddServer(pSrv,true)) 
						delete pSrv; 
					else AddLogLine(true,GetResString(IDS_SERVERADDED)+"%s", pSrv->GetListName());
				}
				break;
			default:
				break;
			}
			delete pLink;
		} catch(...) {
			AddLogLine(true, GetResString(IDS_LINKNOTADDED));
		}
	}
	return true;
}
#endif

#if 0
LRESULT CxmuleDlg::OnFileHashed(WPARAM wParam,LPARAM lParam){
	CKnownFile* result = (CKnownFile*)lParam;
	if (wParam){
		CPartFile* requester = (CPartFile*)wParam;
		requester->PartFileHashFinished(result);
	}
	else{
		theApp.sharedfiles->filelist->SafeAddKFile(result);
		theApp.sharedfiles->SafeAddKFile(result);	
	}
	return true;
}
#endif

void CxmuleDlg::OnHashingShutdown(wxCommandEvent&)
{
	if ( m_app_state != APP_STATE_SHUTINGDOWN ) {
		printf("Hashing thread ended, restarting\n");
		CAddFileThread::Setup();
	} else {
		printf("Hashing thread terminated, ready to shutdown\n");
		Destroy();
	}
}

void CxmuleDlg::OnClose(wxCloseEvent& evt)
{
  if (evt.CanVeto() && theApp.glob_prefs->IsConfirmExitEnabled() )
    if (wxNO==wxMessageBox(GetResString(IDS_MAIN_EXIT),GetResString(IDS_MAIN_EXITTITLE),wxYES_NO)) {
      evt.Veto();
      return;
    }

  // we are going DOWN
  m_app_state=APP_STATE_SHUTINGDOWN;

  theApp.OnlineSig(); // Added By Bouc7 

  // Close sockets to avoid new clients coming in
  theApp.listensocket->StopListening();
  theApp.clientudp->Destroy();
  theApp.serverconnect->Disconnect();

  // Signal the hashing thread to terminate
  CAddFileThread::Shutdown();

  // TODO: Add your message handler code here and/or call default
  //WINDOWPLACEMENT wp;wp.length=sizeof(wp);GetWindowPlacement(&wp);
  //theApp.glob_prefs->SetWindowLayout(wp);
  
  // saving data & stuff
  theApp.knownfiles->Save();
  transferwnd->downloadlistctrl->SaveSettings(CPreferences::tableDownload);
  transferwnd->uploadlistctrl->SaveSettings(CPreferences::tableUpload);
  transferwnd->queuelistctrl->SaveSettings(CPreferences::tableQueue);
//  if(searchwnd->searchlistctrl) searchwnd->searchlistctrl->SaveSettings(CPreferences::tableSearch);
  sharedfileswnd->sharedfilesctrl->SaveSettings(CPreferences::tableShared);
  serverwnd->serverlistctrl->SaveSettings(CPreferences::tableServer);
  
  theApp.glob_prefs->Add2TotalDownloaded(theApp.stat_sessionReceivedBytes);
  theApp.glob_prefs->Add2TotalUploaded(theApp.stat_sessionSentBytes);

  // save friends
  theApp.friendlist->SaveList();

  theApp.glob_prefs->Save();
  
  transferwnd->downloadlistctrl->DeleteAllItems();
  //xmuledlg->chatwnd->chatselector->DeleteAllItems();
  theApp.clientlist->DeleteAll();
//  if(searchwnd->searchlistctrl) searchwnd->searchlistctrl->DeleteAllItems();

  // Hint: Destroying the window is delayed until the hashing thread
  //       does signal its termination.
//  Destroy();
  //CTrayDialog::OnCancel();
}

#if 0
void CxmuleDlg::OnTrayRButtonDown(CPoint pt){
	if (trayPopup) return;
	trayPopup.CreatePopupMenu(); 

	UINT flagsC;
	UINT flagsD;
	
	// set connect/disconnect to enabled and disabled
	if (!theApp.serverconnect->IsConnected()) {
		flagsD=MF_STRING || MF_DISABLED; flagsC=MF_STRING;
	} else {
		flagsC=MF_STRING || MF_DISABLED; flagsD=MF_STRING;
	}

	trayPopup.AddMenuTitle((CString)"xMule v"+(CString)VERSION);

	trayPopup.AppendMenu(	MF_STRING ,MP_RESTORE, GetResString(IDS_MAIN_POPUP_RESTORE) ); 
	trayPopup.SetDefaultItem(MP_RESTORE);
	trayPopup.AppendMenu(MF_SEPARATOR);

	trayPopup.AppendMenu(flagsC,MP_CONNECT, GetResString(IDS_CONNECTTOANYSERVER));
	trayPopup.AppendMenu(flagsD ,MP_DISCONNECT,   GetResString(IDS_MAIN_BTN_DISCONNECT)); 

	trayPopup.AppendMenu(MF_STRING,MP_EXIT, GetResString(IDS_EXIT)); 
	SetForegroundWindow();
	trayPopup.TrackPopupMenu(TPM_LEFTALIGN |TPM_RIGHTBUTTON, pt.x, pt.y, this); 

	trayPopup.DestroyMenu();

}
#endif

void CxmuleDlg::StartConnection(){
	AddLogLine(true, GetResString(IDS_CONNECTING));
	theApp.serverconnect->ConnectToAnyServer();
	ShowConnectionState(false);
}

void CxmuleDlg::CloseConnection(){
	theApp.serverconnect->Disconnect();
	theApp.OnlineSig(); // Added By Bouc7 
}
void CxmuleDlg::RestoreWindow(){
  //if (TrayIsVisible()) TrayHide();	
  //ShowWindow(SW_SHOW);
}

#if 0
void CxmuleDlg::OnBnClicked2(){ SetActiveDialog(&transferwnd); }
void CxmuleDlg::OnBnClicked3(){ SetActiveDialog(&serverwnd);}
void CxmuleDlg::OnBnClicked4(){ SetActiveDialog(&searchwnd);}
void CxmuleDlg::OnBnClicked5(){ SetActiveDialog(&sharedfileswnd);}
void CxmuleDlg::OnBnClicked6(){ SetActiveDialog(&chatwnd);}
void CxmuleDlg::OnBnClicked7(){ SetActiveDialog(&statisticswnd);}
void CxmuleDlg::OnBnClicked8(){ preferenceswnd.DoModal();}
void CxmuleDlg::OnBnClicked9(){ SetActiveDialog(&ircwnd);}
#endif

void CxmuleDlg::ShowStatistics() {
  	statisticswnd->ShowStatistics();	
}

void CxmuleDlg::UpdateTrayIcon(int procent)
{
	//HICON mytrayIcon;
	//if (mytrayIcon!=NULL) DestroyIcon(mytrayIcon);

	// set the limits of where the bar color changes (low-high)
	int pLimits16[1] = {100};

	// set the corresponding color for each level
	COLORREF pColors16[1] = {theApp.glob_prefs->GetStatsColor(11)};

#if 0
	// start it up
	if (theApp.serverconnect->IsConnected())
		trayIcon.Init(sourceTrayIcon,100,1,1,16,16,RGB(21,97,110));
	else trayIcon.Init(sourceTrayIconGrey,100,1,1,16,16,RGB(21,97,110));
#endif

	// load our limit and color info
	m_wndTaskbarNotifier->SetColorLevels(pLimits16,pColors16,1);

	// generate the icon (destroy these icon using DestroyIcon())
	int pVals16[1] = {procent};

#if 0
	mytrayIcon = trayIcon.Create(pVals16);
	ASSERT (mytrayIcon != NULL);
	if (mytrayIcon)
		m_wndTaskbarNotifier->TraySetIcon(mytrayIcon,true);
	//TrayUpdate();
#endif
	// ei hienostelua. tarvii kuitenki pelleill gtk:n kanssa 
	char** data;
	if(!theApp.serverconnect)
	  data=mule_Tr_grey_ico;
	else {
	  if (theApp.serverconnect->IsConnected())
		if(!theApp.serverconnect->IsLowID()) {
 	      data=mule_TrayIcon_ico;
 	    } else {
 	      data=mule_Tr_yellow_ico;
 	    }

	  else data=mule_Tr_grey_ico;
	}

	m_wndTaskbarNotifier->TraySetIcon(data,true,pVals16);
}
#if 0
int CxmuleDlg::OnCreate(LPCREATESTRUCT lpCreateStruct){
	CTrayDialog::OnCreate(lpCreateStruct);
	/*if (theApp.glob_prefs->UseSplashScreen()){
		ModifyStyleEx(0,WS_EX_LAYERED);
		SetLayeredWindowAttributes(0, 0, LWA_ALPHA);
	}*/
	return 0;

}
#endif

//START - enkeyDEV(kei-kun) -TaskbarNotifier-
void CxmuleDlg::ShowNotifier(wxString Text, int MsgType, bool ForceSoundOFF) {
#if 0
	bool ShowIt = false;
	switch(MsgType) {
		case TBN_CHAT:
            if (theApp.glob_prefs->GetUseChatNotifier()) {
				m_wndTaskbarNotifier->Show(Text, MsgType);
				ShowIt = true;
			}
			break;
		case TBN_DLOAD:
            if (theApp.glob_prefs->GetUseDownloadNotifier()) {
				m_wndTaskbarNotifier->Show(Text, MsgType);
				ShowIt = true;			
			}
			break;
		case TBN_LOG:
            if (theApp.glob_prefs->GetUseLogNotifier()) {
				m_wndTaskbarNotifier->Show(Text, MsgType);
				ShowIt = true;			
			}
			break;
		case TBN_IMPORTANTEVENT:
			if (theApp.glob_prefs->GetNotifierPopOnImportantError()) {
				m_wndTaskbarNotifier->Show(Text, MsgType);
				ShowIt = true;			
			}
			break;	// added by InterCeptor (bugfix) 27.11.02
		case TBN_NEWVERSION:
			if (theApp.glob_prefs->GetNotifierPopOnNewVersion()) {
				m_wndTaskbarNotifier->Show(Text, MsgType);
				ShowIt = true;			
			}
			break;
		case TBN_NULL:
            m_wndTaskbarNotifier->Show(Text, MsgType);
			ShowIt = true;			
			break;
	}
	
#if 0
    if (theApp.glob_prefs->GetUseSoundInNotifier() && !ForceSoundOFF && ShowIt == true)
        PlaySound(theApp.glob_prefs->GetNotifierWavSoundPath(), NULL, SND_FILENAME | SND_NOSTOP | SND_NOWAIT | SND_ASYNC);
#endif
#endif
}


#if 0
LRESULT CxmuleDlg::OnTaskbarNotifierClicked(WPARAM wParam,LPARAM lParam)
{
	int msgType = TBN_NULL;
	msgType = m_wndTaskbarNotifier.GetMessageType();
    switch(msgType)
	{
	case TBN_CHAT:
		RestoreWindow();
		SetActiveDialog(&chatwnd);
		break;
	case TBN_DLOAD:
		RestoreWindow();
		SetActiveDialog(&transferwnd);
	//	{TODO: aprire la cartella incoming?}
		break;
	case TBN_LOG:
		RestoreWindow();
		SetActiveDialog(&serverwnd);	
		break;
	}
    return 0;
}
#endif
//END - enkeyDEV(kei-kun) -TaskbarNotifier-

void CxmuleDlg::Localize(){
#if 0
	GetDlgItem(IDC_BN_DOWNLOADS)->SetWindowText(GetResString(IDS_EM_TRANS));
	GetDlgItem(IDC_BN_SERVERS)->SetWindowText(GetResString(IDS_EM_SERVER));
	GetDlgItem(IDC_BN_SEARCH)->SetWindowText(GetResString(IDS_EM_SEARCH));
	GetDlgItem(IDC_BN_FILES)->SetWindowText(GetResString(IDS_EM_FILES));
	GetDlgItem(IDC_BN_MESSAGES)->SetWindowText(GetResString(IDS_EM_MESSAGES));
	GetDlgItem(IDC_BN_STATISTICS)->SetWindowText(GetResString(IDS_EM_STATISTIC));
	GetDlgItem(IDC_BN_PREFERENCES)->SetWindowText(GetResString(IDS_EM_PREFS));
#endif
}

void CxmuleDlg::OnUDPTimer(wxTimerEvent& evt)
{
  if(!IsRunning()) 
    return;
  theApp.serverlist->SendNextPacket();
}

void CxmuleDlg::OnSocketTimer(wxTimerEvent& evt)
{
  if(!IsRunning())
    return;

	CServerConnect *_this= theApp.serverconnect; 
	_this->StopConnectionTry();
	if (_this->IsConnected()) return; 

	_this->ConnectToAnyServer(); 
}

void CxmuleDlg::OnQLTimer(wxTimerEvent& evt)
{
  printf("QLTimer\n");
  if(!IsRunning())
    return;
  CQueueListCtrl::QueueUpdateTimer();
}
